#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved
import argparse
import pathlib
import sys

project_root = str(pathlib.Path(__file__).resolve().parent.parent)
if project_root not in sys.path:
    sys.path.insert(0, project_root)
from importers.common.log_util import my_logger, logger
from importers.meta.meta_create import *


def parse_args():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-m',
                        help='必填参数. 创建事件-create_event,创建事件属性-create_event_variables,创建用户属性-create_user_variables,'
                             '创建主体字段-create_item_variables,绑定事件与事件属性-bind_event_variables,导入元数据-export_meta，导出元数据-import_meta',
                        required=True, metavar="")
    parser.add_argument('-ik', '--item_key', help='创建主体时必填参数:指定主体标识符', default="", metavar="")
    parser.add_argument('-k', '--key', help='必填参数. 需要创建事件名', default="", metavar="")
    parser.add_argument('-t', '--type', help='创建事件/用户属性必填参数. 数据类型-(string,int,double)', default="",
                        metavar="")
    parser.add_argument('-a', '--attr', help='绑定事件与属性必填参数，多个属性名使用英文逗号分隔', metavar="")
    parser.add_argument('-n', '--name', help='可选参数. 事件显示名', default="", metavar="")
    parser.add_argument('-d', '--desc', help='可选参数. 事件描述', default="", metavar="")
    parser.add_argument('-f', '--file', help='导入/导出元数据必填参数. 文件名', metavar="")
    parser.add_argument('-analy', '--analysis', help='用于创建分析主体，TRUE-创建分析主体,FALSE-创建主体字典表.',
                        default=False)
    args = parser.parse_args()
    return args.__dict__


def execute_importer(args):
    m = args.get("m")
    token = get_token()
    analysis = str(args.get('analysis')).upper()
    if 'TRUE'.__eq__(analysis):
        analysis = True
    elif 'FALSE'.__eq__(analysis):
        analysis = False
    else:
        logger.error("[-analy/--analysis]参数值不对")
        exit(-1)
    args['analysis'] = analysis

    if 'create_event'.__eq__(m):
        key = args.get('key')
        check_key(key)
        create_info = create_event(token, key, args.get('name'), args.get('desc'))
        my_logger.info("创建事件成功", create_info)
    elif 'create_event_variables'.__eq__(m):
        key = args.get('key')
        check_key(key)
        create_info = create_event_variables(token, key, args.get('type'), args.get('name'), args.get('desc'))
        my_logger.info("创建事件属性成功", create_info)
    elif 'create_user_variables'.__eq__(m):
        key = args.get('key')
        check_key(key)
        create_info = create_user_variables(token, key, args.get('type'), args.get('name'), args.get('desc'))
        my_logger.info("创建用户属性成功", create_info)
    elif 'create_item'.__eq__(m):
        item_key = args.get('item_key')
        check_key(item_key)
        create_info = create_item(token, item_key, args['analysis'], args.get('name'), args.get('desc'))
        my_logger.info("创建主体成功", create_info)
    elif 'create_item_variables'.__eq__(m):
        item_key = args.get('item_key')
        check_key(item_key)
        import_item_variables(token, item_key, args['analysis'], args.get('file'))
    elif 'bind_event_variables'.__eq__(m):
        key = args.get('key')
        check_key(key)
        bind_info, key_list = bind_event_variables(token, key, args.get('name'), args.get('attr'))
        my_logger.info("成功绑定事件属性:{},info:{}".format(key_list, bind_info))
    elif 'export_meta'.__eq__(m):
        export_meta(token, args.get('file'))
    elif 'import_meta'.__eq__(m):
        import_meta(token, args.get('file'))
    else:
        logging.error("请确认填写项目名！")
        exit(-1)


def do_meta(params):
    my_logger.info("Meta Data Task Start")
    # 设置默认参数值
    default_params = {
        'm': '',
        'item_key': '',
        'key': '',
        'type': '',  # 默认为空字符串
        'attr': '',  # 默认为空字符串
        'name': '',  # 默认为空字符串
        'desc': '',  # 默认为空字符串
        'file': '',  # 默认为空字符串
        'analysis': False
    }
    combined_params = {**default_params, **params}

    execute_importer(combined_params)
    my_logger.info("Meta Data Task Finish")


def main():
    args = parse_args()
    execute_importer(args)


if __name__ == '__main__':
    my_logger.info("Meta Data Task Start")
    main()
    my_logger.info("Meta Data Task Finish")



#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved
import argparse
import pathlib
import sys

project_root = str(pathlib.Path(__file__).resolve().parent.parent)
if project_root not in sys.path:
    sys.path.insert(0, project_root)
from importers.db_import.database_import import events, user_variables, item_variables
from importers.common.log_util import logger, my_logger
from importers.meta.meta_create import *


def parse_args():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-m',
                        help='必填参数. 事件events，用户属性user_variables，主体数据导入item_variables.',
                        required=True,
                        type=str)
    parser.add_argument('-ds', '--datasource_id',
                        help='必填参数. 数据源ID.',
                        required=True,
                        type=str)
    parser.add_argument('-item_key',
                        help='必填参数. item_key',
                        required=False,  # 根据-m参数判断是否必填
                        default='',
                        type=str)
    parser.add_argument('-f', '--format',
                        help='可选参数. 导入数据格式,目前支持mysql、hive格式.',
                        type=str,
                        metavar="")
    parser.add_argument('-db_host', '--host',
                        help='必填参数. 客户数据源地址',
                        metavar="")
    parser.add_argument('-db_user', '--user',
                        help='必填参数. 客户数据源用户',
                        type=str,
                        metavar="")
    parser.add_argument('-db_pass', '--password',
                        help='必填参数. 客户数据源密码',
                        type=str,
                        metavar="")
    parser.add_argument('-db_port', '--port',
                        help='必填参数. 客户数据源端口',
                        metavar="")
    parser.add_argument('-sql', '--sql',
                        help='必选参数. sql语句',
                        type=str,
                        metavar="")
    parser.add_argument('-b', '--batch',
                        help='可选参数. hive模式下每批次处理多少条数据',
                        type=int,
                        default=100000,
                        metavar="")
    parser.add_argument('-s', '--start_time',
                        help='用户行为数据必选参数. 开始时间',
                        metavar="")
    parser.add_argument('-e', '--end_time',
                        help='用户行为数据必选参数. 结束时间',
                        metavar="")
    parser.add_argument('-j', '--jobName',
                        help='指定导入任务名称',
                        metavar="")
    parser.add_argument('-c', '--clear',
                        help='可选参数. True-导入数据成功后清理掉FTP上数据,False-导入数据成功后不清理掉FTP上数据.',
                        default=False)
    args = parser.parse_args()
    return args.__dict__


def execute_importer(args):
    tunnels = getTunnels()
    m = args.get('m')
    if args.get('datasource_id') not in tunnels:
        logger.error("数据源不存在")
        exit(-1)
    args['datasource_id'] = [args.get('datasource_id'), tunnels[args.get('datasource_id')]]
    if 'events'.__eq__(m):
        events(args)
    elif 'user_variables'.__eq__(m):
        user_variables(args)
    elif 'item_variables'.__eq__(m):
        item_variables(args)
    else:
        logging.error("请确认填写项目名！")
        exit(-1)


def do_importer(params):
    my_logger.info("Data Importer Start")
    # 设置默认参数值
    default_params = {
        'm': None,  # 必填参数, 没有默认值
        'datasource_id': None,  # 必填参数, 没有默认值
        'item_key': '',  # 默认为空字符串
        'format': None,  # 可选参数, 没有默认值
        'host': None,  # 必填参数, 没有默认值
        'user': None,  # 必填参数, 没有默认值
        'password': None,  # 必填参数, 没有默认值
        'port': None,  # 必填参数, 没有默认值
        'sql': None,  # 必选参数, 没有默认值
        'batch': 100000,  # 默认为 100000
        'start_time': None,  # 用户行为数据必选参数, 没有默认值
        'end_time': None,  # 用户行为数据必选参数, 没有默认值
        'jobName': None,  # 可选参数, 没有默认值
        'clear': False,  # 默认为 False
    }

    combined_params = {**default_params, **params}
    # 如果 params 中有相同的键，则 params 的值会覆盖 default_params 中的默认值
    execute_importer(combined_params)
    my_logger.info("Data Importer Finish")


def main():
    args = parse_args()
    execute_importer(args)


if __name__ == '__main__':
    my_logger.info("Data Importer Start")
    main()
    my_logger.info("Data Importer Finish")



#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved
import csv
import json

import pandas as pd

from importers.data_import.data_model import UserVariablesSv, UserVariablesJson
from importers.data_import.data_format_util import *
from importers.common import http_util
from importers.common.common_util import getVariables
from importers.common.log_util import logger, my_logger
from json.decoder import JSONDecodeError
from importers.meta.data_center import getdataCenterUserVariables, getImportJobStatus, trigger_job


def user_variables_import(args):
    """
       用户属性导入，按数据格式处理
    """
    # Step one: 校验事件数据基础参数，并预处理
    # 1. 数据源是否为属性
    ds = args.get('datasource_id')
    if 'USER_PROPERTY' in ds[1]:
        args['datasource_id'] = ds[1]['USER_PROPERTY']
    else:
        logger.error("数据源不属于用户属性类型")
        exit(-1)
    # Step one: 按数据格式处理
    f = str(args.get('format'))
    if 'JSON'.__eq__(f):
        user_variables_import_send(
            UserVariablesJson(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                              datasourceId=args.get('datasource_id'), jobName=args.get('jobName'),
                              clear=args.get('clear'))
        )
    elif 'CSV'.__eq__(f):
        separator = args.get("separator")
        separator = ',' if separator == '' else separator
        user_variables_import_send(
            UserVariablesSv(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                            datasourceId=args.get('datasource_id'), jobName=args.get('jobName')
                            , attributes=args.get('attributes'), separator=separator,
                            skipHeader=args.get('skip_header'), clear=args.get('clear'))
        )
    elif 'TSV'.__eq__(f):
        separator = args.get("separator")
        separator = '\t' if separator == '' else separator
        user_variables_import_send(
            UserVariablesSv(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                            datasourceId=args.get('datasource_id'), jobName=args.get('jobName'),
                            attributes=args.get('attributes'), separator=separator, skipHeader=args.get('skip_header'),
                            clear=args.get('clear'))
        )


def sv_import_prepare_process(attributes, paths, skip_header, separator, qualifier):
    """
      1.校验数据基本信息
      2.SV格式数据转换为Json格式导入
    """
    # Step 1: 校验有无attributes,有无重复列名
    if attributes is None:
        logger.error(f"[-attr/--attributes]参数值不存在")
        exit(-1)

    cols = str(attributes).split(',')
    duplicate_col = check_sv_col_duplicate(cols)
    if duplicate_col is not None:
        logger.error(f"[-attr/--attributes]出现重复列值[{duplicate_col}]")
        exit(-1)

    keys = getVariables(getdataCenterUserVariables())
    for path in paths:
        count = 0
        if skip_header:
            replace_first_line_with_header(path, cols, separator)
        # 读取文件第一行，与预期表头比较
        with open(path, 'r', encoding='utf8') as f:
            first_line = f.readline().strip()
            expected_header = separator.join(cols)
            if first_line != expected_header:
                add_header_to_file(path, cols, separator)

        skip_header = True
        with open(path, 'r', encoding='utf8') as f:
            for line in f:
                count += 1
                # Step 2: 数据列是否一致
                line = line.replace('\n', '').replace('\\t', '\t')
                line_normal = True
                if check_sv_header_col_count(cols, line.split(separator)) is False:
                    if not check_csv_header_col_count(cols, line, separator):
                        remove_first_line_from_csv(path)
                        logger.error(f"第{count}行:导入文件[{path}]的列数和参数值列数不一致\n请检查文件分隔符，并通过参数[-sep]指定")
                        return False
                if not line_normal:
                    continue
                # Step 3: 校验数据列是否一致
        df = pd.read_csv(path, sep=separator, header=0 if skip_header else None,
                         names=cols if not skip_header else None, quotechar=qualifier)
        # 校验表头
        if skip_header and not all(col in df.columns for col in cols):
            logger.error(f"文件[{path}]的列名与预期不匹配")
            exit(-1)
        # 逐行处理数据
        for index, row in df.iterrows():
            col_value = row.to_dict()
            # 处理 userKey 逻辑
            user_key = col_value.get('userKey', '')
            if 'userKey' in cols and user_key == '$notuser':
                logger.error(f"文件[{path}]数据[{row.to_string()}]中的userKey值不合法")
                exit(-1)
            # 构建属性字典
            attrs = {key: value for key, value in col_value.items() if
                     len(str(value)) != 0 and key not in ['userKey', 'userId']}
            # 校验用户属性是否在 GIO 平台定义
            undefined_keys = [key for key in attrs if not key.startswith('$') and key not in keys]
            if undefined_keys:
                logger.error(f"文件[{path}]数据[{row.to_string()}]用户属性{undefined_keys}在GIO平台未定义")
                exit(-1)
    return True


def user_variables_import_send(userVariables):
    """
       用户属性导入，Json格式数据处理
    """
    # Step 1: 执行Debug
    f = userVariables.format

    if userVariables.debug:
        if 'JSON'.__eq__(f):
            if user_variables_debug_process(userVariables.path) is not True:
                logger.error("Debug校验未通过")
                exit(-1)
        else:
            if sv_import_prepare_process(attributes=userVariables.attributes,
                                         paths=userVariables.path,
                                         skip_header=userVariables.skipHeader,
                                         separator=userVariables.separator,
                                         qualifier=userVariables.qualifier) is not True:
                logger.error("Debug校验未通过")
                exit(-1)

    if len(str(userVariables.jobName)) == 0 or userVariables.jobName is None:
        jobName = f"Python_user_{time.strftime('%Y%m%d_%H%M%S', time.localtime(time.time()))}"
    else:
        jobName = userVariables.jobName
    # Step 2: 创建导入任务
    if 'JSON'.__eq__(f):
        job_info = create_task_json(userVariables.datasourceId, jobName)
    else:
        job_info = create_task_sv(userVariables.datasourceId, jobName, userVariables.separator)
    # 任务名重复时，获取不到job信息时，程序直接结束
    if job_info is None:
        logger.error("job_info为空，无法创建导入任务")
        exit(-1)
    else:
        my_logger.info(f"创建导入任务: {job_info}")
    direct = job_info['argument']['directory']
    # Step 3: 上传数据到FTP
    my_logger.info(f"文件开始上传至FTP")
    put_file(userVariables.path, direct)

    # Step 4: 启动导入任务
    start_time = time.time()
    trigger_job(job_info['id'])
    my_logger.info(f"开始执行导入任务")
    flag = True
    while flag:
        eventImportJob = getImportJobStatus(job_info['id'])
        if eventImportJob is not None:
            stage = eventImportJob['stage']
            error = eventImportJob['error']
            if stage is not None and stage.__eq__("FINISH"):
                end_time = time.time()
                cost_time = end_time - start_time
                my_logger.info("导入成功")
                delete_file(userVariables.path, direct)
                my_logger.info("执行导入任务耗时:%.3f秒" % cost_time)
                flag = False
            elif stage is not None and stage.__eq__("ERROR"):
                end_time = time.time()
                cost_time = end_time - start_time
                if error is not None:
                    message = error.get('message', 'No message available')
                else:
                    message = 'Error object is None'
                logger.error(f"导入失败,错误信息为[ {message} ] \n FTP文件路径: {direct}")
                my_logger.info("执行导入任务耗时:%.3f秒" % cost_time)
                exit(-1)
        if flag:
            my_logger.info(f"等待任务完成......")
            time.sleep(10)


def create_task_json(ds, name):
    """
           创建任务,允许用户自定义更改任务名称
        """
    if len(str(name)) == 0:
        body = '''{ "operationName": "createEventImportJob", "variables": { "fileType":"ftp", "timeRange":"", 
        "tunnelId": "%s", "createType":"PYTHON" }, "query": "mutation createEventImportJob($tunnelId: HashId!, 
        $timeRange: String, $fileType: String, $createType: String) { createEventImportJob(tunnelId: $tunnelId, 
        timeRange: $timeRange,fileType: $fileType, createType: $createType) { id name argument { directory __typename 
        } __typename } }" }''' % ds
    else:
        body = '''{ "operationName":"createEventImportJob", "variables":{ "fileType":"ftp", "tunnelId":"%s", 
        "timeRange":"", "name":"%s", "createType":"PYTHON" }, "query":"mutation createEventImportJob($name: String, 
        $tunnelId: HashId!, $timeRange: String, $fileType: String, $createType: String) { createEventImportJob(name: 
        $name, tunnelId: $tunnelId, timeRange: $timeRange, fileType: $fileType, createType: $createType) { id name 
        argument { directory __typename } __typename } }" }''' % (ds, name)
    resp = http_util.send_graphql_post(http_util.get_token(), body.encode('utf-8'))
    try:
        return resp['createEventImportJob']
    except TypeError:
        logger.error("自定义任务名称已存在！")


def create_task_sv(ds, name, separator):
    """
           创建任务,允许用户自定义更改任务名称
    """
    separator = separator.replace("\t", "\\\\t")
    if len(str(name)) == 0:
        body = '''{ "operationName": "createEventImportJobV2", "variables": { "fileType":"csv", "timeRange":"", 
        "tunnelId": "%s", "createType":"PYTHON", "csvSeparator":"%s" }, "query": "mutation createEventImportJobV2(
        $tunnelId: HashId!, $timeRange: String, $fileType: String, $createType: String, $csvSeparator: String) { 
        createEventImportJobV2(tunnelId: $tunnelId, timeRange: $timeRange,fileType: $fileType, createType: 
        $createType, csvSeparator: $csvSeparator) { id name argument { directory __typename } __typename } }" }''' % (
            ds, separator)
    else:
        body = '''{ "operationName":"createEventImportJobV2", "variables":{ "fileType":"csv", "tunnelId":"%s", 
        "timeRange":"", "name":"%s", "createType":"PYTHON", "csvSeparator":"%s" }, "query":"mutation 
        createEventImportJobV2($name: String, $tunnelId: HashId!, $timeRange: String, $fileType: String, $createType: 
        String, $csvSeparator: String) { createEventImportJobV2(name: $name, tunnelId: $tunnelId, timeRange: 
        $timeRange, fileType: $fileType, createType: $createType, csvSeparator:$csvSeparator) { id name argument { 
        directory __typename } __typename } }" }''' % (ds, name, separator)
    resp = http_util.send_graphql_post(http_util.get_token(), body.encode('utf-8'))
    try:
        return resp['createEventImportJobV2']
    except TypeError:
        logger.error("自定义任务名称已存在！")


def user_variables_debug_process(paths):
    """
       用户属性导入Debug
       1、校验有无userId
       2、校验用户属性(条件:是否是平台内置和是否定义)
    """
    keys = getVariables(getdataCenterUserVariables())
    count = 0
    error_count = 0
    for path in paths:
        with open(path, 'r', encoding='utf8') as f:
            for line in f:
                count = count + 1
                line = line.replace('\n', '')
                if not line == '':
                    normal = True
                    error_message = ""
                    try:
                        data_dictionary = json.loads(line)
                        # userId或anonymousId
                        if 'userId' not in data_dictionary:
                            normal = False
                            error_message += f"userId不存在\n"

                        if 'userKey' in data_dictionary:
                            if data_dictionary['userKey'] == '$notuser':
                                normal = False
                                error_message += f"用户属性导入不支持用户身份为‘$notuser’\n"

                        # 用户属性
                        if 'attrs' in data_dictionary:
                            if not isinstance(data_dictionary['attrs'], dict):
                                normal = False
                                error_message += f"attrs数据格式不对\n"

                            for key in data_dictionary['attrs']:
                                if data_dictionary['attrs'][key] is None:
                                    normal = False
                                    error_message += f"用户属性[{key}]的值为NULL,请检查原始数据\n"

                                if key not in keys:
                                    normal = False
                                    error_message += f"用户属性[{key}]在GIO平台未定义\n"

                    except JSONDecodeError:
                        normal = False
                        error_message += f"非JSON格式\n"

                    if not normal:
                        logger.error(f"第{count}行:文件[{path}]数据[{line}],\n"
                                     f"{error_message}")
                        error_count += 1
                    if count % 500000 == 0:
                        my_logger.info(f"已经校验{count}条数据......")
                else:
                    logger.warn(f"第{count}行为空，跳过该行")
        f.close()

    if error_count == 0:
        my_logger.info(f"本次共校验[{count}]行数据")
        return True
    else:
        my_logger.info(f"本次共校验失败[{error_count}]行数据")
        return False



"""
SaaS元数据
"""
import json

from importers.common.config_util import SaasConfig
from importers.common.http_util import send_rest_get


def getSaasMeta(file):
    """
      导出SaaS元数据
     """
    events = 'v1/api/projects/' + SaasConfig.uid + '/dim/events'
    event_vars = 'v1/api/projects/' + SaasConfig.uid + '/vars/events'
    user_vars = 'v1/api/projects/' + SaasConfig.uid + '/vars/peoples'
    anonymous_var = 'v1/api/projects/' + SaasConfig.uid + '/vars/visitors'
    parmas = {'project_uid': SaasConfig.uid}
    events_context = send_rest_get(events, parmas)
    event_var_context = send_rest_get(event_vars, parmas)
    usr_var_context = send_rest_get(user_vars, parmas)
    anonymous_var_context = send_rest_get(anonymous_var, parmas)
    # 事件
    event_list = []
    result = {}
    for data in events_context:
        event_res = {}
        event_res['key'] = data['key'].strip()
        event_res['name'] = data['name'].strip()
        event_res['description'] = data['description'].strip()
        event_list.append(event_res)
    result['events'] = event_list
    # 事件属性
    evar_list = []
    for evar in event_var_context:
        evar_res = {}
        evar_res['key'] = evar['key'].strip()
        evar_res['name'] = evar['name'].strip()
        evar_res['valueType'] = str(evar['type']).lower().strip()
        evar_res['description'] = ''
        evar_list.append(evar_res)
    result['event_variables'] = evar_list
    # 用户属性
    uvar_list = []
    uvar_set = set()
    for uvar in usr_var_context:
        uvar_res = {}
        uvar_res['key'] = uvar['key'].strip()
        uvar_res['name'] = uvar['name'].strip()
        uvar_res['valueType'] = 'string'
        uvar_res['description'] = uvar['description'].strip()
        uvar_list.append(uvar_res)
        uvar_set.add(uvar['key'].strip())
    uvar_size = len(uvar_set)

    # 访问用户属性
    avar_list = []
    avar_set = set()
    for avar in anonymous_var_context:
        avar_res = {}
        avar_res['key'] = avar['key'].strip()
        avar_res['name'] = avar['name'].strip()
        avar_res['valueType'] = 'string'
        avar_res['description'] = avar['description'].strip()
        avar_list.append(avar_res)
        avar_set.add(avar['key'].strip())
    avar_size = len(avar_set)

    var_set = set()
    for var in avar_set:
        if var not in uvar_set:
            var_set.add(var)
    if uvar_size + avar_size == len(uvar_set | var_set):
        result['user_variables'] = uvar_list + avar_list
    else:
        for i in avar_list:
            if i['key'] in var_set:
                uvar_list.append(i)
        result['user_variables'] = uvar_list

    attr_list = []
    for data in events_context:
        event_res = {}
        list = []
        for attr in data['attrs']:
            item = {}
            item['key'] = attr['key'].strip()
            list.append(item)
        event_res['key'] = data['key'].strip()
        event_res['attributes'] = list
        attr_list.append(event_res)
    result['bind_event_variables'] = attr_list
    json_str = str(result).replace('\'', '\"')
    with open(file, "w+", encoding='utf-8') as f:
        f.write(json_str.encode('utf-8').decode('utf-8'))
    print("事件，事件属性，用户属性导出成功~")



# !/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved


class ImportBase:
    """
      Import Base
    """

    def __init__(self, name, path, debug, format, datasourceId, jobName, clear):
        self.name = name
        self.path = path
        self.debug = debug
        self.format = format
        self.datasourceId = datasourceId
        self.jobName = jobName
        self.clear = clear


class SVBase:
    """
      SV Base
    """

    def __init__(self, skipHeader, separator, qualifier='"'):
        self.qualifier = qualifier
        self.separator = separator
        self.skipHeader = skipHeader


class UserVariables(ImportBase):
    """
      用户属性
    """

    def __init__(self, name, path, debug, format, datasourceId, jobName, clear):
        ImportBase.__init__(self, name, path, debug, format, datasourceId, jobName, clear)


class UserVariablesJson(UserVariables):
    """
      用户属性-Josn格式
    """

    def __init__(self, name, path, format, datasourceId, jobName, debug=True, clear=False):
        UserVariables.__init__(self, name, path, debug, format, datasourceId, jobName, clear)


class UserVariablesSv(UserVariables, SVBase):
    """
      用户属性-CSV/TSV格式
    """

    def __init__(self, name, path, debug, format, datasourceId, jobName, attributes, skipHeader, separator, clear,
                 qualifier='"'):
        UserVariables.__init__(self, name, path, debug, format, datasourceId, jobName, clear)
        SVBase.__init__(self, skipHeader, separator, qualifier)
        self.attributes = attributes


class Events(ImportBase):
    """
      用户行为
    """

    def __init__(self, name, path, debug, format, datasourceId, eventStart, eventEnd, jobName, clear):
        ImportBase.__init__(self, name, path, debug, format, datasourceId, jobName, clear)
        self.eventStart = eventStart
        self.eventEnd = eventEnd
        self.jobName = jobName
        self.clear = clear


class EventsJson(Events):
    """
      用户行为-Josn格式
    """

    def __init__(self, name, path, format, datasourceId, eventStart, eventEnd, jobName, debug=True, clear=False):
        Events.__init__(self, name, path, debug, format, datasourceId, eventStart, eventEnd, jobName, clear)


class EventsCv(Events, SVBase):
    """
      用户行为-CSV/TSV格式
    """

    def __init__(self, name, path, debug, format, datasourceId, eventStart, eventEnd, jobName, attributes, skipHeader,
                 separator, clear, qualifier='"'):
        Events.__init__(self, name, path, debug, format, datasourceId, eventStart, eventEnd, jobName, clear)
        SVBase.__init__(self, skipHeader, separator, qualifier)
        self.attributes = attributes


class DataEvent:
    """
      用户行为大数据规定格式
    """

    def __init__(self, userId, event, timestamp, attrs, userKey='', eventId=None, dataSourceId=None):
        self.userId = userId
        self.event = event
        self.userKey = userKey
        self.eventId = eventId
        self.timestamp = timestamp
        self.attrs = attrs
        self.dataSourceId = dataSourceId


class DataUser:
    """
      用户属性大数据规定格式
    """

    def __init__(self, userId, userKey, attrs):
        self.userId = userId
        self.userKey = userKey
        self.attrs = attrs


class DataItem:
    """
      主体大数据规定格式
    """

    def __init__(self, item_id, attrs):
        self.item_id = item_id
        self.attrs = attrs


class ImportItem:
    """
      Import Item
    """

    def __init__(self, name, path, debug, format, datasourceId, itemKey, jobName, clear):
        self.name = name
        self.path = path
        self.debug = debug
        self.format = format
        self.datasourceId = datasourceId
        self.itemKey = itemKey
        self.jobName = jobName
        self.clear = clear


class ItemVariables(ImportItem):
    """
      主体
    """

    def __init__(self, name, path, debug, format, datasourceId, itemKey, jobName, clear):
        ImportItem.__init__(self, name, path, debug, format, datasourceId, itemKey, jobName, clear)
        super().__init__(name, path, debug, format, datasourceId, itemKey, jobName, clear)


class ItemVariablesJson(ItemVariables):
    """
      主体-Json格式
    """

    def __init__(self, name, path, format, datasourceId, itemKey, jobName, debug=True, clear=False):
        ItemVariables.__init__(self, name, path, debug, format, datasourceId, itemKey, jobName, clear)


class ItemVariablesSv(ItemVariables, SVBase):
    """
      主体-TSV格式
    """

    def __init__(self, name, path, debug, format, datasourceId, itemKey, jobName, attributes, skipHeader, separator, clear,
                 qualifier='"'):
        ItemVariables.__init__(self, name, path, debug, format, datasourceId, itemKey, jobName, clear)
        SVBase.__init__(self, skipHeader, separator, qualifier)
        self.attributes = attributes


class ItemVariablesCsv(ItemVariables):
    """
      主体-Csv格式
    """

    def __init__(self, name, path, format, datasourceId, itemKey, jobName, debug=True, clear=False):
        ItemVariables.__init__(self, name, path, debug, format, datasourceId, itemKey, jobName, clear)



from setuptools import setup, find_packages


install_requires = [
]

setup(
    name='importer',
    version='1.0',
    author='GrowingIO',
    packages=find_packages(include=["importer*"]),
    install_requires=install_requires
)



import json
import os
import tempfile
import time

from importers.common.common_util import time_str_to_timestamp_of_tz, getItemVariables
from importers.common.config_util import BaseConfig
from importers.common.log_util import logger
from importers.data_import.data_events import ONE_DAY_MILLISECOND, events_import_send
from importers.db_import.hive_import import event_hive_import, user_hive_import, item_hive_import
from importers.db_import.mysql_import import event_mysql_import, user_mysql_import, item_mysql_import
from importers.meta.data_center import getdataCenterItemModels


def events(args):
    try:
        start = time_str_to_timestamp_of_tz(args.get('start_time'), '%Y-%m-%d', BaseConfig.timezone) * 1000
        end = time_str_to_timestamp_of_tz(args.get('end_time'), '%Y-%m-%d',
                                          BaseConfig.timezone) * 1000 + ONE_DAY_MILLISECOND
    except TypeError and ValueError:
        logger.error("[-s/--start_time]或[-e/--end_time]时间参数格式不对,格式为:YYYY-MM-DD")
        exit(-1)
    ds = args.get('datasource_id')
    if 'HISTORY_EVENT' in ds[1]:
        args['datasource_id'] = ds[1]['HISTORY_EVENT']
    else:
        logger.error("数据源不属于用户行为类型")
        exit(-1)
    f = args.get('format')
    if 'MYSQL'.__eq__(str(f).upper()):
        event_mysql_import(args, start, end)
    elif 'HIVE'.__eq__(str(f).upper()):
        event_hive_import(args, start, end)
    else:
        logger.error("format不在取值范围内")
        exit(-1)


def user_variables(args):
    ds = args.get('datasource_id')
    if 'USER_PROPERTY' in ds[1]:
        args['datasource_id'] = ds[1]['USER_PROPERTY']
    else:
        logger.error("数据源不属于用户属性类型")
        exit(-1)
    if 'MYSQL'.__eq__(str(args.get('format')).upper()):
        user_mysql_import(args)
    elif 'HIVE'.__eq__(str(args.get('format')).upper()):
        user_hive_import(args)
    else:
        logger.error("format不在取值范围内")
        exit(-1)


def item_variables(args):
    keys = getItemVariables(getdataCenterItemModels())
    itemKey = args.get('item_key')
    ds = args.get('datasource_id')
    if 'HISTORY_ITEM' in ds[1]:
        args['datasource_id'] = ds[1]['HISTORY_ITEM']
    else:
        logger.error("数据源不属于主体类型")
        exit(-1)
    if itemKey not in keys:
        logger.error(f"item_Key主体标识符[{itemKey}]不存在")
        exit(-1)

    if 'MYSQL'.__eq__(str(args.get('format')).upper()):
        item_mysql_import(args)
    elif 'HIVE'.__eq__(str(args.get('format')).upper()):
        item_hive_import(args)
    else:
        logger.error("format不在取值范围内")
        exit(-1)



#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved
import argparse
import os
import pathlib
import sys
import logging

sys.path.append(str(pathlib.Path(os.path.abspath(__file__)).parent.parent))
from importers.saas_export.saas_meta import getSaasMeta


def parse_args():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-m',
                        help='必填参数. 创建事件-create_event,创建事件属性-create_event_variables,创建用户属性-create_user_variables,绑定事件与事件属性-bind_event_variables,导入元数据-export_meta，导出元数据-import_meta',
                        required=True, metavar="")
    parser.add_argument('-f', '--file', help='导入/导出元数据必填参数. 文件名', metavar="")
    args = parser.parse_args()
    return args.__dict__


def main():
    args = parse_args()
    m = args.get("m")
    if 'export_saas_meta'.__eq__(m):
        getSaasMeta(args.get("file"))
    else:
        logging.error("请确认填写项目名！")
        exit(-1)


if __name__ == '__main__':
    main()



import json

from importers.common.http_util import send_graphql_post, get_token
from importers.common.log_util import my_logger
from importers.meta.check_util import *
from importers.meta.data_center import getMetaData, getBindEvent

"""
元数据创建
"""


def create_event(token, key, name, desc):
    """
       创建事件
    """
    names = check_name(name, key)
    flag, var, id_list = check_event_attr_exsit(token, key)
    str_sub = ''
    if len(id_list) != 0:
        attr_str = ''
        for i in range(len(id_list)):
            attr_str += '{\"type\":\"EVENT_VARIABLE\",\"id\":\"' + str(id_list[i]) + '\"},'
        str_sub = attr_str[:len(attr_str) - 1]
    if flag:
        if desc == '' and name != '':
            body = """{"operationName":"updateDataCenterCustomEvent","variables":{"customEvent":{"attributes":[%s],"valueType":"counter","description":"%s","key":"%s","name":"%s"},
                   "id":"%s"},"query":"mutation updateDataCenterCustomEvent($id: HashId!, $customEvent: CustomEventInput!) {updateDataCenterCustomEvent(id: $id, customEvent: $customEvent) 
                   {name    __typename}}"}""" % (str_sub, var['description'], key, var['name'], var['id'])
        elif names == key and desc != '':
            body = """{"operationName":"updateDataCenterCustomEvent","variables":{"customEvent":{"attributes":[%s],"valueType":"counter","description":"%s","key":"%s","name":"%s"},
                   "id":"%s"},"query":"mutation updateDataCenterCustomEvent($id: HashId!, $customEvent: CustomEventInput!) {updateDataCenterCustomEvent(id: $id, customEvent: $customEvent) 
                   {name    __typename}}"}""" % (str_sub, desc, key, var['name'], var['id'])
        elif name != '' and desc != '':
            body = """{"operationName":"updateDataCenterCustomEvent","variables":{"customEvent":{"attributes":[%s],"valueType":"counter","description":"%s","key":"%s","name":"%s"},
                   "id":"%s"},"query":"mutation updateDataCenterCustomEvent($id: HashId!, $customEvent: CustomEventInput!) {updateDataCenterCustomEvent(id: $id, customEvent: $customEvent) 
                   {name    __typename}}"}""" % (str_sub, desc, key, names, var['id'])
        else:
            body = """{"operationName":"updateDataCenterCustomEvent","variables":{"id":"%s","customEvent":{"name":"%s","key":"%s","description":"%s",
             "valueType":"counter","attributes":[%s]}},"query":"mutation updateDataCenterCustomEvent($id: HashId!, $customEvent: CustomEventInput!) 
             {  updateDataCenterCustomEvent(id: $id, customEvent: $customEvent) {    name    __typename  }}"}""" % (
                var['id'], var['name'], key, var['description'], str_sub)
    else:
        body = """{"operationName":"createDataCenterCustomEvent","variables":{"customEvent":{"attributes":[],"valueType":"counter",
            "key":"%s","name":"%s","description":"%s"}},"query":"mutation createDataCenterCustomEvent($customEvent: CustomEventInput!) 
            {createDataCenterCustomEvent(customEvent: $customEvent) {name    __typename }}"}""" % (key, name, desc)
    content = send_graphql_post(token, body.encode('utf-8'))
    return content


# 创建事件属性
def create_event_variables(token, key, valuetype, name, desc):
    """
       创建事件属性
    """
    name = check_name(name, key)
    check_event_valuetype(valuetype)
    flag, var = check_event_var_exsit(token, key)
    if flag:
        if desc == '':
            body = """{"operationName":"updateDataCenterEventVariable","variables":{"id":"%s","eventVariable":{"name":"%s","key":"%s",
                    "description":"%s","valueType":"%s"}},"query":"mutation updateDataCenterEventVariable($id: HashId!, $eventVariable: VariableInput!) 
                    {  updateDataCenterEventVariable(id: $id, eventVariable: $eventVariable) {    name    __typename  }}"}""" % (
                var['id'], name, key, var['description'], var['valueType'])
        elif name == key:
            body = """{"operationName":"updateDataCenterEventVariable","variables":{"id":"%s","eventVariable":{"name":"%s","key":"%s",
                     "description":"%s","valueType":"%s"}},"query":"mutation updateDataCenterEventVariable($id: HashId!, $eventVariable: VariableInput!) 
                     {  updateDataCenterEventVariable(id: $id, eventVariable: $eventVariable) {    name    __typename  }}"}""" % (
                var['id'], var['name'], key, desc, var['valueType'])
        else:
            body = """{"operationName":"updateDataCenterEventVariable","variables":{"id":"%s","eventVariable":{"name":"%s","key":"%s",
                     "description":"%s","valueType":"%s"}},"query":"mutation updateDataCenterEventVariable($id: HashId!, $eventVariable: VariableInput!) 
                     {  updateDataCenterEventVariable(id: $id, eventVariable: $eventVariable) {    name    __typename  }}"}""" % (
                var['id'], name, key, desc, var['valueType'])
        content = send_graphql_post(token, body.encode('utf-8'))
        my_logger.info("事件属性存在且更新成功")
    else:
        body = """{"operationName":"createDataCenterEventVariable","variables":{"eventVariable":{"name":"%s","key":"%s",
        "valueType":"%s","description":"%s"}},"query":"mutation createDataCenterEventVariable($eventVariable: VariableInput!) 
        {createDataCenterEventVariable(eventVariable: $eventVariable) {name    __typename}}"}""" % (
            name, key, valuetype, desc)
        content = send_graphql_post(token, body.encode('utf-8'))
    return content


# 创建用户属性
def create_user_variables(token, key, valuetype, name, desc):
    """
       创建用户属性
    """
    name = check_name(name, key)
    check_user_valuetype(valuetype)
    flag, var = check_user_var_exsit(token, key)
    if flag:
        if desc == '':
            body = """{"operationName":"updateDataCenterUserVariable","variables":{"id":"%s","userVariable":{"name":"%s","key":"%s","description":"%s",
            "valueType":"%s"}},"query":"mutation updateDataCenterUserVariable($id: HashId!, $userVariable: VariableInput!) {  updateDataCenterUserVariable(id: $id,
             userVariable: $userVariable) {    name    __typename  }}"}""" % (
                var['id'], name, key, var['description'], var['valueType'])
        elif name == key:
            body = """{"operationName":"updateDataCenterUserVariable","variables":{"id":"%s","userVariable":{"name":"%s","key":"%s","description":"%s",
             "valueType":"%s"}},"query":"mutation updateDataCenterUserVariable($id: HashId!, $userVariable: VariableInput!) {  updateDataCenterUserVariable(id: $id,
             userVariable: $userVariable) {    name    __typename  }}"}""" % (
                var['id'], var['name'], key, desc, var['valueType'])
        else:
            body = """{"operationName":"updateDataCenterUserVariable","variables":{"id":"%s","userVariable":{"name":"%s","key":"%s","description":"%s",
            "valueType":"%s"}},"query":"mutation updateDataCenterUserVariable($id: HashId!, $userVariable: VariableInput!) {  updateDataCenterUserVariable(id: $id,
            userVariable: $userVariable) {    name    __typename  }}"}""" % (
                var['id'], name, key, desc, var['valueType'])
        content = send_graphql_post(token, body.encode('utf-8'))
        my_logger.info("用户属性存在且更新成功")
    else:
        body = """{"operationName":"createDataCenterUserVariable","variables":{"userVariable":{"name":"%s","key":"%s","valueType":"%s","description":"%s"}},
        "query":"mutation createDataCenterUserVariable($userVariable: VariableInput!) {createDataCenterUserVariable(userVariable: $userVariable) {
            id    name    __typename  }}"}""" % (name, key, valuetype, desc)
        content = send_graphql_post(token, body.encode('utf-8'))
    return content


def bind_event_variables(token, key, name, attr):
    """
        绑定事件与事件属性
    """
    id_list, key_list = check_attr(token, attr)
    flag, var, attr_id_list = check_event_attr_exsit(token, key)
    name = check_name(name, key)
    attr_str = ''
    size = len(id_list)
    for i in range(size):
        attr_str += '{\"type\":\"EVENT_VARIABLE\",\"id\":\"' + str(id_list[i]) + '\"},'
    str_sub = attr_str[:len(attr_str) - 1]
    if flag and len(attr_id_list) > 0:
        body = """{"operationName":"updateDataCenterCustomEvent","variables":{"customEvent":{"attributes":[%s],"valueType":"counter","description":"%s","key":"%s","name":"%s"},
        "id":"%s"},"query":"mutation updateDataCenterCustomEvent($id: HashId!, $customEvent: CustomEventInput!) {updateDataCenterCustomEvent(id: $id, customEvent: $customEvent)
         {   name  __typename  }}"}""" % (str_sub, var['description'], key, var['name'], var['id'])
    elif flag and len(attr_id_list) == 0:
        body = """{"operationName":"updateDataCenterCustomEvent","variables":{"customEvent":{"attributes":[%s],"valueType":"counter","description":"%s","key":"%s","name":"%s"},
        "id":"%s"},"query":"mutation updateDataCenterCustomEvent($id: HashId!, $customEvent: CustomEventInput!) {updateDataCenterCustomEvent(id: $id, customEvent: $customEvent) 
        {name    __typename}}"}""" % (str_sub, var['description'], key, var['name'], var['id'])
    else:
        body = """{"operationName":"createDataCenterCustomEvent","variables":{"customEvent":{"attributes":[%s],"valueType":"counter",
        "key":"%s","name":"%s"}},"query":"mutation createDataCenterCustomEvent($customEvent: CustomEventInput!) {createDataCenterCustomEvent(customEvent: $customEvent) {
        name    __typename }}"}""" % (str_sub, key, name)
    content = send_graphql_post(token, body.encode('utf-8'))
    return content, key_list


def export_meta(token, file):
    """
       导出元数据
    """
    meta_data = getMetaData(token)
    meta_data['events'] = meta_data.pop('dataCenterCustomEvents')
    meta_data['event_variables'] = meta_data.pop('dataCenterEventVariables')
    meta_data['user_variables'] = meta_data.pop('dataCenterUserVariables')
    user_attr_list = list()
    for i in meta_data['user_variables']:
        if not str(i['key']).startswith("$"):
            user_attr_list.append(i)
    meta_data['user_variables'] = user_attr_list

    event_list = list()
    for i in meta_data['events']:
        if not str(i['key']).startswith("$"):
            event_list.append(i)
    meta_data['events'] = event_list

    event_attr_list = list()
    for i in meta_data['event_variables']:
        if not str(i['key']).startswith("$"):
            event_attr_list.append(i)
    meta_data['event_variables'] = event_attr_list
    meta_data = str(meta_data).replace('\'', '\"').replace('\\', '\\\\')[:-1]
    event = getBindEvent(token)
    event_vars_list = list()
    event['bind_event_variables'] = event.pop('dataCenterCustomEvents')
    for i in event['bind_event_variables']:
        if not str(i['key']).startswith("$") and len(i['attributes']) != 0:
            event_vars_list.append(i)
    event['bind_event_variables'] = event_vars_list
    event = str(event).replace('\'', '\"').replace('\\', '\\\\')[1:]
    json_str = meta_data + ',' + event
    with open(file, 'w+', encoding='utf-8') as f:
        f.write(json_str.encode('utf-8').decode('utf-8'))
        # json.dump(json.loads(json_str), f)
    my_logger.info("导出元数据成功~")


def import_meta(token, file):
    """
       导入元数据
    """
    with open(file, 'r', encoding='utf-8') as f:
        json_loads = json.loads(f.read())
        if len(json_loads['events']) != 0:
            for event in json_loads['events']:
                check_key(event['key'])
                create_event(token, event['key'], event['name'], event['description'])
        if len(json_loads['event_variables']) != 0:
            for var in json_loads['event_variables']:
                check_key(var['key'])
                create_event_variables(token, var['key'], str(var['valueType']).lower(), var['name'],
                                       var['description'])
        if len(json_loads['user_variables']) != 0:
            for var in json_loads['user_variables']:
                check_key(var['key'])
                create_user_variables(token, var['key'], str(var['valueType']).lower(), var['name'], var['description'])
        if len(json_loads['bind_event_variables']) != 0:
            for var in json_loads['bind_event_variables']:
                attr_str = ''
                for attr in var['attributes']:
                    attr_str += str(attr['key']) + ','
                sub_attr = attr_str[:-1]
                if len(sub_attr) > 2:
                    bind_event_variables(token, var['key'], '', sub_attr)
    my_logger.info("导入元数据成功~")


def create_item(token, item_key, analysis, name, desc):
    """
       创建主体
    """
    names = check_name(name, item_key)
    check_item_key(item_key)
    desc = check_desc(desc)
    flag, var_id, var_name, var_desc = check_item_var_exsit(token, item_key)
    if name != var_name:
        check_item_name_exsit(token, name)
    if flag:
        if desc == '' and name != '':
            body = """{"operationName":"updateDataCenterItemModel","variables":{"itemModel":{"description":"%s","name":"%s"},
                        "id":"%s"},"query":"mutation updateDataCenterItemModel($id: HashId!, $itemModel: ItemModelInput!)
                        {\n  updateDataCenterItemModel(id: $id, itemModel: $itemModel) {\n    name\n    __typename\n  }\n}\n"}""" \
                   % (var_desc, name, var_id)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("主体存在且更新成功")
        elif names == item_key and desc != '':
            body = """{"operationName":"updateDataCenterItemModel","variables":{"itemModel":{"description":"%s","name":"%s"},
                                    "id":"%s"},"query":"mutation updateDataCenterItemModel($id: HashId!, $itemModel: ItemModelInput!)
                                    {\n  updateDataCenterItemModel(id: $id, itemModel: $itemModel) {\n    name\n    __typename\n  }\n}\n"}""" \
                   % (desc, var_name, var_id)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("主体存在且更新成功")
        elif names == item_key and desc == '':
            body = """{"operationName":"updateDataCenterItemModel","variables":{"itemModel":{"description":"%s","name":"%s"},
                                    "id":"%s"},"query":"mutation updateDataCenterItemModel($id: HashId!, $itemModel: ItemModelInput!)
                                    {\n  updateDataCenterItemModel(id: $id, itemModel: $itemModel) {\n    name\n    __typename\n  }\n}\n"}""" \
                   % (var_desc, var_name, var_id)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("主体存在且更新成功")
        else:
            body = """{"operationName":"updateDataCenterItemModel","variables":{"itemModel":{"description":"%s","name":"%s"},
            "id":"%s"},"query":"mutation updateDataCenterItemModel($id: HashId!, $itemModel: ItemModelInput!)
            {\n  updateDataCenterItemModel(id: $id, itemModel: $itemModel) {\n    name\n    __typename\n  }\n}\n"}""" \
                   % (desc, name, var_id)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("主体存在且更新成功")
    else:
        check_item_name_exsit(token, name)
        if analysis:
            body = """{"operationName":"createDataCenterItemModel","variables":{"itemModel":{"attributes":[{
            "valueType":"string","description":"%s","isPrimaryKey":true,"key":"%s","name":"%s"}],
            "description":"%s","name":"%s", "analysis": true}}, "query":"mutation createDataCenterItemModel($itemModel: 
            ItemModelInput!) {\n  createDataCenterItemModel(itemModel: $itemModel) {\n    name\n    id\n    
            __typename\n  }\n}\n"}""" % (
                desc, item_key, name, desc, name)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("分析主体创建成功:", content)
        else:
            body = """{"operationName":"createDataCenterItemModel","variables":{"itemModel":{"attributes":[{
            "valueType":"string","description":"%s","isPrimaryKey":true,"key":"%s","name":"%s"}],"description":"%s",
            "name":"%s"}}, "query":"mutation createDataCenterItemModel($itemModel: ItemModelInput!) {\n  
            createDataCenterItemModel(itemModel: $itemModel) {\n    name\n    id\n    __typename\n  }\n}\n"}""" % (
                desc, item_key, name, desc, name)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("字典表创建成功:", content)
    return content


def import_item_variables(token, item_key, analysis, file):
    """
       导入主体字段
    """
    flag, var_id, var_name, var_desc = check_item_var_exsit(token, item_key)
    if flag:
        with open(file, 'r', encoding='utf-8') as f:
            json_loads = json.loads(f.read())
            for item in json_loads['attributes']:
                create_item(token, item_key, analysis, json_loads['name'], json_loads['description'])
                create_item_variables(token, var_id, item['key'], item['name'], item['valueType'], item['description'])
    else:
        with open(file, 'r', encoding='utf-8') as f:
            json_loads = json.loads(f.read())
            name = json_loads['name']
            desc = json_loads['description']
            create_item(token, item_key, analysis, name, desc)
            flag, var_id, var_name, var_desc = check_item_var_exsit(token, item_key)
            for item in json_loads['attributes']:
                check_key(item['key'])
                create_item_variables(token, var_id, item['key'], item['name'], item['valueType'], item['description'])


def create_item_variables(token, old_id, key, name, valueType, desc):
    check_item_key_variables(key)
    names = check_name(name, key)
    desc = check_desc(desc)
    flag, var_name, var_valueType, var_desc, new_id = check_item_var_key_exsit(token, old_id, key)
    if name != var_name:
        check_item_name_exsit(token, name)
    if flag:
        if valueType != var_valueType:
            logging.error("主体属性<{}>类型不可更新".format(key))
            exit(-1)
        if desc == '' and name != '':
            body = """{"operationName":"updateDataCenterItemVariable",
            "variables":{"itemVariable":{"description":"%s","valueType":"%s","key":"%s","name":"%s"},
            "id":"%s"},"query":"mutation updateDataCenterItemVariable($id: HashId!, $itemVariable: VariableInput!) 
            {\n  updateDataCenterItemVariable(id: $id, itemVariable: $itemVariable) {\n    name\n    __typename\n  }\n}\n"}
            """ % (var_desc, var_valueType, key, name, new_id)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("主体属性存在且更新成功")
        elif names == key and desc != '':
            body = """{"operationName":"updateDataCenterItemVariable",
                        "variables":{"itemVariable":{"description":"%s","valueType":"%s","key":"%s","name":"%s"},
                        "id":"%s"},"query":"mutation updateDataCenterItemVariable($id: HashId!, $itemVariable: VariableInput!) 
                        {\n  updateDataCenterItemVariable(id: $id, itemVariable: $itemVariable) {\n    name\n    __typename\n  }\n}\n"}
                        """ % (desc, var_valueType, key, var_name, new_id)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("主体属性存在且更新成功")
        else:
            body = """{"operationName":"updateDataCenterItemVariable",
                                    "variables":{"itemVariable":{"description":"%s","valueType":"%s","key":"%s","name":"%s"},
                                    "id":"%s"},"query":"mutation updateDataCenterItemVariable($id: HashId!, $itemVariable: VariableInput!) 
                                    {\n  updateDataCenterItemVariable(id: $id, itemVariable: $itemVariable) {\n    name\n    __typename\n  }\n}\n"}
                                    """ % (desc, var_valueType, key, name, new_id)
            content = send_graphql_post(token, body.encode('utf-8'))
            my_logger.info("主体属性存在且更新成功")
    else:
        check_item_name_exsit(token, name)
        check_item_key_exsit(token, key)
        check_item_valuetype(valueType)
        body = """{"operationName":"createDataCenterItemVariable","variables"
                    :{"attribute":{"isPrimaryKey":false,"description":"%s","valueType":"%s","key":"%s","name":"%s"}
                    ,"id":"%s"},"query":"mutation createDataCenterItemVariable($id: HashId!, $attribute: ItemVariableInput!) 
                    {\n  dataCenterAddItemModelAttribute(id: $id, attribute: $attribute) {\n    name\n    __typename\n  }\n}\n"}
                    """ % (desc, valueType, key, name, old_id)
        content = send_graphql_post(token, body.encode('utf-8'))
        my_logger.info("主体属性创建成功:", content)

    return content



#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved
import argparse
import pathlib
import sys

project_root = str(pathlib.Path(__file__).resolve().parent.parent)
if project_root not in sys.path:
    sys.path.insert(0, project_root)
from importers.common.log_util import my_logger
from importers.meta.meta_create import *
from importers.clear_data.clear_data import clearUserData, clearUserMeta


def parse_args():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-m',
                        help='必填参数. 触发用户删除任务-clear_users，批量添加待删除用户-clear_users_meta',
                        required=True, metavar="")
    parser.add_argument('-n', '--now',
                        help='可选参数. True-立即执行离线任务,false-天任务执行清理任务')
    parser.add_argument('-u', '--users',
                        help='可选参数, 添加待删除用户', type=str)
    args = parser.parse_args()
    return args.__dict__


def execute_importer(args):
    m = args.get("m")
    if 'clear_users'.__eq__(m):
        clearUserData(args.get("now"))
    elif 'clear_users_meta'.__eq__(m):
        clearUserMeta(args.get("users"))
    else:
        logging.error("请确认填写项目名！")
        exit(-1)



def do_user(params):
    my_logger.info("Clear User Task Start")
    execute_importer(params)
    my_logger.info("Clear User Task Finish")

def main():
    args = parse_args()
    execute_importer(args)


if __name__ == '__main__':
    my_logger.info("Clear User Task Start")
    main()
    my_logger.info("Clear User Task Finish")



import json
import os
import time

from pyhive.exc import OperationalError
from importers.common.common_util import get_all_file, remove_file, hive_connect, getVariables
from importers.common.config_util import get_temp_dir_from_config
from importers.common.log_util import logger, my_logger
from importers.data_import.data_events import events_import_send
from importers.data_import.data_format_util import validate_data_event
from importers.data_import.data_item_variable import item_variables_import_send
from importers.data_import.data_model import EventsJson, DataEvent, DataUser, UserVariablesJson, ItemVariablesJson, \
    DataItem
from importers.data_import.data_user_variable import user_variables_import_send
from importers.meta.data_center import getBindEvent, getdataCenterUserVariables, getdataCenterEventVariables


def event_hive_import(args, start, end):
    """
       行为数据导入，HIVE数据源
    """
    try:
        conn = hive_connect(host=args.get('host'),
                            port=int(args.get('port')),
                            user=args.get('user')
                            )
    except TypeError:
        logger.error("HIVE连接失败。")
        exit(-1)
    cursor = conn.cursor()
    try:
        sql = args.get('sql')
        cursor.execute(sql)
        my_logger.info(sql)
    except OperationalError:
        logger.error("请检查SQL语句")
        exit(-1)
    desc = cursor.description
    desc_list = []
    for d in desc:
        desc_list.append(d[0])
    if 'event' not in desc_list or 'timestamp' not in desc_list:
        logger.error("event或timestamp字段不存在")
        exit(-1)
        # 检查userId字段是否存在，如果不存在，则检查userKey字段
    if 'userid' not in desc_list and 'userkey' not in desc_list:
        logger.error("缺少userid需指定\n若传主体事件,则数据需字段userKey,且值为‘$notuser’")
        exit(-1)

    temp_dir = get_temp_dir_from_config()  # 从配置中获取临时存储目录
    current_tmp_path = os.path.join(temp_dir, str(int(round(time.time() * 1000))))
    if os.path.exists(current_tmp_path) is False:
        os.makedirs(current_tmp_path)
    my_logger.info(f"临时存储Json文件目录：[{current_tmp_path}]")
    json_file_abs_path = current_tmp_path + '/' + 'tmp_events.json'
    event = getBindEvent()
    cstm_keys = {}
    for i in event['dataCenterCustomEvents']:
        list = []
        for a in i['attributes']:
            list.append(a['key'])
        cstm_keys[i['key']] = list
    cstm_attr_keys = getVariables(getdataCenterEventVariables())
    try:
        start_time = time.time()
        wf = open(json_file_abs_path, 'w')
        cnt = 0
        count = 0
        while True:
            cnt += 1
            data = cursor.fetchmany(size=args.get('batch'))
            if len(data) == 0 and cnt == 1:
                logger.error("数据为空")
                exit(-1)
            elif len(data) == 0 and cnt > 1:
                my_logger.info(f"临时文件总共写入{count}条数据")
                end_time = time.time()
                cost_time = end_time - start_time
                my_logger.info("读取SQL数据,写入临时文件耗时:%.3f秒" % cost_time)
                if len(str(args.get('jobName'))) == 0 or args.get('jobName') is None:
                    job_name = f"Python_events_{time.strftime('%Y%m%d_%H%M%S', time.localtime(time.time()))}"
                else:
                    job_name = args.get('jobName')
                events_import_send(
                    EventsJson(name='events',
                               path=get_all_file(current_tmp_path),
                               format='JSON',
                               debug=False,
                               eventStart=start,
                               eventEnd=end,
                               datasourceId=args.get('datasource_id'),
                               jobName=job_name,
                               clear=False
                               )
                )
                break
            else:
                for row in data:
                    tmp = {}
                    var = {}
                    userId_present = True
                    for a in range(len(row)):
                        if desc_list[a] == 'event' or desc_list[a] == 'timestamp':
                            tmp[desc_list[a]] = row[a]
                        elif desc_list[a] == 'userid':
                            userId_present = False
                            tmp['userId'] = row[a]
                        elif desc_list[a] == 'eventid' and row[a] != '':
                            tmp['eventId'] = row[a]
                        elif desc_list[a] == 'userkey' and row[a] != '':
                            tmp['userKey'] = row[a]
                        else:
                            var[desc_list[a]] = row[a]
                    tmp['attrs'] = var
                    # 检查userId字段，如果不存在则检查userKey是否为$notuser
                    if userId_present:
                        if tmp['userKey'] == '$notuser':
                            tmp['userId'] = ''
                        else:
                            logger.error("导入主体事件时,userKey的值不是$notuser")
                            exit(-1)
                    if 'eventId' in tmp:
                        eventId = tmp['eventId']
                    else:
                        eventId = None
                    if 'userKey' in tmp:
                        userKey = tmp['userKey']
                    else:
                        userKey = ''
                    data_event = DataEvent(userId=tmp['userId'], event=tmp['event'],
                                           timestamp=tmp['timestamp'], attrs=tmp['attrs'],
                                           userKey=userKey, eventId=eventId)
                    is_valid, error_message = validate_data_event(data_event, start, end, cstm_keys,
                                                                  cstm_attr_keys)
                    if not is_valid:  # 异常
                        logger.error(f"{error_message}")
                        exit(-1)
                    wf.write(json.dumps(data_event.__dict__, ensure_ascii=False))
                    wf.write('\n')
                    count += 1
                    if count % 2000000 == 0:
                        my_logger.info(f"已经写入{count}条数据进临时文件......")
                wf.flush()

    finally:
        remove_file(current_tmp_path)


def user_hive_import(args):
    """
       用户属性导入，HIVE数据源
    """
    try:
        conn = hive_connect(host=args.get('host'),
                            port=int(args.get('port')),
                            user=args.get('user'))
    except TypeError:
        logger.error("HIVE连接失败。")
        exit(-1)
    cursor = conn.cursor()
    try:
        sql = args.get('sql')
        cursor.execute(sql)
        my_logger.info(sql)
    except OperationalError:
        logger.error("请检查SQL语句")
        exit(-1)
    desc = cursor.description
    desc_list = []
    for d in desc:
        desc_list.append(d[0])
    if 'userid' not in desc_list:
        logger.error("userId字段不存在")
        exit(-1)

    temp_dir = get_temp_dir_from_config()  # 从配置中获取临时存储目录
    current_tmp_path = os.path.join(temp_dir, str(int(round(time.time() * 1000))))
    if os.path.exists(current_tmp_path) is False:
        os.makedirs(current_tmp_path)
    my_logger.info(f"临时存储Json文件目录：[{current_tmp_path}]")
    keys = getVariables(getdataCenterUserVariables())
    json_file_abs_path = current_tmp_path + '/' + 'tmp_user.json'
    start_time = time.time()
    try:
        wf = open(json_file_abs_path, 'w')
        cnt = 0
        count = 0
        while True:
            cnt += 1
            data = cursor.fetchmany(size=args.get('batch'))
            if len(data) == 0 and cnt == 1:
                logger.error("数据为空")
                exit(-1)
            elif len(data) == 0 and cnt > 1:
                my_logger.info(f"临时文件总共写入{count}条数据")
                end_time = time.time()
                cost_time = end_time - start_time
                my_logger.info("读取SQL数据,写入临时文件耗时:%.3f秒" % cost_time)
                if len(str(args.get('jobName'))) == 0 or args.get('jobName') is None:
                    job_name = f"Python_user_{time.strftime('%Y%m%d_%H%M%S', time.localtime(time.time()))}"
                else:
                    job_name = args.get('jobName')
                user_variables_import_send(
                    UserVariablesJson(name='user_variables',
                                      path=get_all_file(current_tmp_path),
                                      debug=False,
                                      format='JSON',
                                      datasourceId=args.get('datasource_id'),
                                      jobName=job_name,
                                      clear=False)
                )
                break
            else:
                json_list = []
                for row in data:
                    res = {}
                    var = {}
                    for a in range(len(row)):
                        if desc_list[a] == 'userid':
                            res['userId'] = row[a]
                        elif desc_list[a] == 'userkey' and row[a] != '':
                            res['userKey'] = row[a]
                        else:
                            var[desc_list[a]] = row[a]
                    res['attrs'] = var
                    json_list.append(res)
                for json_str in json_list:
                    for key in json_str['attrs']:
                        if key not in keys and key.startswith("$") is False:
                            logger.error("用户属性{}在GIO平台未定义".format(key))
                            exit(-1)
                    if 'userKey' in json_str:
                        userKey = json_str['userKey']
                        if userKey == '$notuser':
                            logger.error("用户属性导入不支持用户身份为‘$notuser’")
                            exit(-1)
                    else:
                        userKey = ''
                    data_event = DataUser(userId=json_str['userId'], userKey=userKey, attrs=json_str['attrs'])
                    wf.write(json.dumps(data_event.__dict__, ensure_ascii=False))
                    wf.write('\n')
                    count += 1
                    if count % 2000000 == 0:
                        my_logger.info(f"已经写入{count}条数据进临时文件......")
                wf.flush()
    finally:
        remove_file(current_tmp_path)


def item_hive_import(args):
    """
       主体导入，HIVE数据源
    """
    try:
        conn = hive_connect(host=args.get('host'),
                            port=int(args.get('port')),
                            user=args.get('user'))
    except TypeError:
        logger.error("HIVE连接失败。")
        exit(-1)
    cursor = conn.cursor()
    try:
        sql = args.get('sql')
        cursor.execute(sql)
        my_logger.info(sql)
    except OperationalError:
        logger.error("请检查SQL语句")
        exit(-1)
    desc = cursor.description
    desc_list = []
    for d in desc:
        desc_list.append(d[0])
    if 'item_id' not in desc_list:
        logger.error("item_id字段不存在")
        exit(-1)

    temp_dir = get_temp_dir_from_config()  # 从配置中获取临时存储目录
    current_tmp_path = os.path.join(temp_dir, str(int(round(time.time() * 1000))))
    if os.path.exists(current_tmp_path) is False:
        os.makedirs(current_tmp_path)
    my_logger.info(f"临时存储Json文件目录：[{current_tmp_path}]")
    json_file_abs_path = current_tmp_path + '/' + args.get('item_key') + '.json'
    start_time = time.time()
    try:
        wf = open(json_file_abs_path, 'w')
        cnt = 0
        count = 0
        while True:
            cnt += 1
            data = cursor.fetchmany(size=args.get('batch'))
            if len(data) == 0 and cnt == 1:
                logger.error("数据为空")
                exit(-1)
            elif len(data) == 0 and cnt > 1:
                my_logger.info(f"临时文件总共写入{count}条数据")
                end_time = time.time()
                cost_time = end_time - start_time
                my_logger.info("读取SQL数据,写入临时文件耗时:%.3f秒" % cost_time)
                if len(str(args.get('jobName'))) == 0 or args.get('jobName') is None:
                    job_name = f"Python_item_{time.strftime('%Y%m%d_%H%M%S', time.localtime(time.time()))}"
                else:
                    job_name = args.get('jobName')
                item_variables_import_send(
                    ItemVariablesJson(name='item_variables',
                                      path=get_all_file(current_tmp_path),
                                      debug=True,
                                      format='JSON',
                                      datasourceId=args.get('datasource_id'),
                                      itemKey=args.get('item_key'),
                                      jobName=job_name,
                                      clear=False)
                )
                break
            else:
                json_list = []
                for row in data:
                    res = {}
                    var = {}
                    for a in range(len(row)):
                        if desc_list[a] == 'item_id' and row[a] != '':
                            res['item_id'] = row[a]
                        else:
                            var[desc_list[a]] = row[a]
                    res['attrs'] = var
                    json_list.append(res)
                for json_str in json_list:
                    data_item = DataItem(item_id=json_str['item_id'], attrs=json_str['attrs'])
                    wf.write(json.dumps(data_item.__dict__, ensure_ascii=False))
                    wf.write('\n')
                    count += 1
                    if count % 2000000 == 0:
                        my_logger.info(f"已经写入{count}条数据进临时文件......")
                wf.flush()
    finally:
        remove_file(current_tmp_path)



import io
import json

import pandas as pd
import numpy as np
import os
import shutil
from pyhive import hive
import pymysql
import pytz
import datetime
import time

from importers.common.config_util import BaseConfig


def split_str(s, sep, qualifier):
    """
    分割复杂字符串
     例如：'"sss,ss", userId,event,tismestamp,,"sss,ss",tismes,"sss,ss"'
     结果：['sss,ss', 'userId', 'event', 'tismestamp', '', 'sss,ss', 'tismes', 'sss,ss']
    :param s: 字符串
    :param sep: 分隔符
    :param qualifier: 文本限定符
    :return: List
    """
    if s is None or s.strip() == '':
        return []
    df = pd.read_csv(io.StringIO(s), sep=sep, quotechar=qualifier, header=None, na_filter=False)
    return np.array(df.where(df.notnull(), '')).tolist()[0]


def get_all_file(path):
    """
    获取目录下全部文件，包含子文件下的
    :param path: 目录或文件路径
    :return: [file_abs_path1,file_abs_path2,...]
    """
    res = []
    if os.path.isdir(path):
        for root, dirs, files in os.walk(path, topdown=False):
            for name in files:
                res.append(os.path.join(root, name))
    else:
        res.append(path)
    return res


def remove_file(path):
    """
     删除Path.如果是文件路径直接删除文件；如果是目录，删除目录及目录下全部
    :param path: 目录或文件
    """
    if os.path.isdir(path):
        shutil.rmtree(path, ignore_errors=True)
    else:
        os.rmdir(path)


def create_dir(dir_str):
    """
     创建目录
    :param dir_str: 目录
    """
    if os.path.exists(dir_str) is False:
        os.makedirs(dir_str)


def time_str_to_timestamp_of_tz(time_str, format, timezone):
    """
     时间字符串转时间戳，带时区
    :param time_str:
    :param format:
    :param timezone:
    :return:
    """
    tz = pytz.timezone(timezone)
    dt = datetime.datetime.strptime(time_str, format)
    t = tz.localize(dt).astimezone(pytz.utc)
    return int(time.mktime(t.utctimetuple())) - time.timezone


def time_format(time_str, timezone):
    """
     支持多种格式的时间字符串转时间戳，带时区
    :param time_str: 时间字符串
    :param timezone: 时区
    :return: time_formatted: 毫秒时间戳
    """
    str_length = len(time_str)
    if time_str.isdigit():
        # 主要是为了支撑一个产品功能，不然无法分辨秒和毫秒
        if str_length == 10:
            time_formatted = int(time_str) * 1000
        elif str_length in [12, 13]:
            time_formatted = int(time_str)

    else:
        if str_length == 10:
            format = get_format("%Y-%m-%d", time_str)
        elif str_length == 19:
            format = get_format("%Y-%m-%d %H:%M:%S", time_str)
        elif str_length == 23:
            format = get_format("%Y-%m-%d %H:%M:%S.%f", time_str)

        time_formatted = time_str_to_timestamp_of_tz(time_str, format, timezone) * 1000

    return time_formatted


def get_format(format, time_str):
    """
     根据传进来的时间来获取具体的时间格式
    :param format: 时间格式
    :param time_str: 时间字符串
    :return: format: 原先时间的格式
    """
    delimiters = ["-", "/"]
    delimiter = time_str[4:5]
    if delimiters.__contains__(delimiter):
        format = format[:2] + delimiter + format[3:5] + delimiter + format[6:]

    return format


def mysql_connect(user, password, host, port):
    """
     获取mysql连接
    :param user: mysql用户名
    :param password: mysql密码
    :param host: mysql Host
    :param port: mysql 端口号
    :return:
    """
    return pymysql.connect(
        host=host,
        port=port,
        user=user,
        charset='utf8',
        passwd=password,
        db='information_schema'
    )


def hive_connect(host, port, user):
    """
     获取hive连接
    :param host: hive Host
    :param port: hive 端口号
    :param user: hive用户名
    :return:
    """
    return hive.Connection(host=host, port=port, username=user, database='default')


def getVariables(dataCenter):
    """
     获取变量标识符列表
    :param dataCenter: getdataCenterEventVariables、getdataCenterUserVariables
    :return:标识符列表
    """
    vars = dataCenter
    var_list = []
    for var in vars:
        var_list.append(var['key'])
    return var_list


def getItemVariables(dataCenter):
    """
     获取主体标识符列表
    :param dataCenter: getdataCenterEventVariables、getdataCenterUserVariables
    :return:标识符列表
    """
    data = json.loads(json.dumps(dataCenter))
    var_list = []
    for item in data:
        for attr in item['attributes']:
            if attr['isPrimaryKey']:
                var_list.append(attr['key'])
    return var_list


def date_range(start_date, end_date, timezone=BaseConfig.timezone):
    """
     遍历一段时间范围，[start_date,end_date)，左闭右开。
    :param start_date 开始时间
    :param end_date 结束时间
    """
    dateTimeFormatter = '%Y-%m-%d'
    time_list = []
    tz = pytz.timezone(timezone)
    start = datetime.datetime.strptime(start_date, dateTimeFormatter).astimezone(tz)
    end = datetime.datetime.strptime(end_date, dateTimeFormatter).astimezone(tz)
    s = start.date()
    e = end.date()
    for i in range((e - s).days):
        day = s + datetime.timedelta(days=i)
        timeArray = time.strptime(str(day), dateTimeFormatter)
        timestamp = (int(time.mktime(timeArray)) - time.timezone) * 1000
        time_list.append(timestamp)
    return time_list



from setuptools import setup, find_packages


with open('importers/README.md', encoding='utf-8') as f:
    long_description = f.read()

install_requires = [
    'pip',
    'numpy',
    'requests',
    'pandas',
    'curlify',
    'PyYAML',
    'psycopg2-binary',
    'pymysql',
    'pyhive',
    'thrift',
    'sasl'
]

setup(
    name='gio_importer',
    version='1.0.6',
    description='GrowingIO Importer是GrowingIO CDP平台元数据创建和数据导入工具',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='gio',
    author_email='dev-growing@startdt.com',
    packages=find_packages(include=["importers*"]),
    include_package_data=True,
    install_requires=install_requires,
    classifiers=[
        # 'Development Status :: 3 - Alpha',  # 发布开发版分类器
        'Development Status :: 5 - Production/Stable',  # 发布稳定版分类器
    ],
    python_requires='>=3.8',
    entry_points={
        'console_scripts': [
            'clear_user=importers.clear_user:main',
            'data_importer=importers.data_importer:main',
            'format_importer=importers.format_importer:main',
            'meta_importer=importers.meta_importer:main',
        ],
    },
)
